\NeedsTeXFormat{LaTeX2e}[1995/12/01]
\ProvidesClass{camc}
[2021/11/26 v0.1: An authoring template for CAMC]

\newif\if@linenumberon\global\@linenumberonfalse%
\newif\if@watermarkon\global\@watermarkonfalse%
\DeclareOption{lineno}{\global\@linenumberontrue}%
\DeclareOption{watermark}{\global\@watermarkontrue}%
\DeclareOption{revise}{\global\@watermarkontrue\global\@linenumberontrue}%
\DeclareOption{final}{\global\@watermarkonfalse\global\@linenumberonfalse}%
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{article}}
\ProcessOptions\relax
\LoadClass{article}
\RequirePackage{etoolbox}

%% ==== !TeX program settings
\RequirePackage{ifxetex,ifluatex}
\newif\ifxetexorluatex
\ifxetex
\xetexorluatextrue
\else
\ifluatex
\xetexorluatextrue
\else
\xetexorluatexfalse
\fi
\fi

%% ==== fonts
\ifxetexorluatex
    \RequirePackage{fontspec}
\else % settings for pdftex
    \RequirePackage[utf8]{inputenc}
    \RequirePackage[T1]{fontenc}
\fi

%\RequirePackage{newtxtext}                            % times new roman font as serif
%\IfFileExists{MyriadPro.sty}{%
%    \RequirePackage{MyriadPro}
%}{%
%    \RequirePackage[defaultsans,scale=0.9]{droidsans} % sans font
%}
%\RequirePackage[defaultmono,scale=0.8]{droidsansmono} % mono font
%\AtEndPreamble{%
%    \RequirePackage{newtxmath}                        % mathfont: times new roman
%    \RequirePackage{microtype}
%}


%% ==== Pagestyles
\def\@springer{Springer}
%\def\@sprlogo{\includegraphics[width=16mm]{springer}}
\def\@jname{Communications on Applied Mathematics and Computation}

% -- ps::opening
\def\@markfont{\sffamily\footnotesize}
\def\rh@rule{\leavevmode\lower6pt\hbox to0pt{\vrule height1pt width\textwidth\hss}}
\def\ps@opening{%
%    \def\@oddhead{\rh@rule\parbox[b]{\textwidth}{\@markfont\@jname\\\@doi}}%
    \def\@oddhead{\rh@rule\parbox[b]{\textwidth}{\@markfont\@jname}}%
    \let\@evenhead\@oddhead
    \def\@oddfoot{}%{\null \hfill \@sprlogo}
    \def\@evenfoot{}%{\@sprlogo \hfill}
    }

% -- ps::headings
\def\myleftmark{\@runningauthor}%
\def\myrightmark{\@runningtitle}%
\def\ps@headings{%
    \def\@oddfoot{}%{\hfill \@sprlogo}%
    \def\@evenfoot{}%{\@sprlogo}
    \def\@evenhead{\rh@rule\hbox{}{\rlap{\@markfont\thepage}}\hfil\@markfont\myleftmark}%
    \def\@oddhead{\rh@rule\@markfont\myrightmark\hfill{\llap{\@markfont\thepage}}}%
}

% apply pagestyle as headings
\pagestyle{headings}


%% ==== Titlepage
% -- env::opening
\AtBeginDocument{%
    \let\title=\title@
    \let\author=\author@
    \let\date=\date@
    \let\doi=\doi@
    \let\received=\received@
    \let\accepted=\accepted@
    \let\revised=\revised@
    \let\published=\published@
    \let\institute=\institute@}
\def\maketitle{%
    \maketitle@@
    \let\title\relax
    \let\author\relax
    \let\date\relax
    \let\doi\relax
    \let\received\relax
    \let\accepted\relax
    \let\revised\relax
    \let\published\relax
    \let\institute\relax}

% -- cmd::doi
\def\@doi{\href{\@doihead\@thedoi}{\@doihead\@thedoi}}
\def\@doihead{https://doi.org/}

\def\@thedoi{10.1007/xxxxx-xxx-xxx-xxxx-x}
\def\doi@#1{%
    \gdef\@thedoi{#1}
}

% -- cmd::raggedrightmargin
% for setting right margin in raggedright mode
\def\raggedrightmargin#1{%
    \let\\\@centercr\@rightskip #1 plus 1fil  \rightskip\@rightskip
    \leftskip\z@skip
    \parindent\z@}

% -- cmd::title
\def\titlecase#1{#1}
\def\titleindent{0pt}
\def\aftertitleskip{10pt}
\def\titleflushstyle{\raggedrightmargin{10mm}} % right margin 10mm for check icon
\def\titlefont{\fontsize{12}{15}\selectfont\bfseries\mathversion{bold}\sffamily}
\def\title@#1{\gdef\@title{\@formattitle{#1}\par \vskip \aftertitleskip }}
\def\@title{}
\def\@formattitle#1{\begingroup
    \def\thanks##1{\global\thankstrue}%
    \setbox\@tempboxa\vbox{#1}
    \endgroup
    \begingroup
    \titleflushstyle
    \ifthanks
    \def\fn##1\thanks##2{\ititle@{##1}{\,\thanks{##2}}}%
    \expandafter\fn#1
    \else
    \def\fn##1{\ititle@{##1}{}}\expandafter\fn\expandafter{#1}%
    \fi
    \endgroup\global\thanksfalse}
\def\titlebaselinefactor{1.05}
\def\ititle@#1#2{\begingroup
    \parindent \titleindent
    \hangindent \titleindent
    \hyphenpenalty10000
    {\titlefont\titlecase{#1}#2%
        \baselineskip=\titlebaselinefactor\baselineskip
        \par}
    \endgroup}

% ---- Authors and Affiliations
% -- cmd::orcid
\let\@orcid\relax
\def\orcid@base{http://orcid.org/}
\protected\def\orcid#1{\gdef\@orcid{#1}}
\IfFileExists{./res/orcid.pdf}
{\def\orcid@pic{\includegraphics{orcid}}}
{\def\orcid@pic{\raise2\p@\hb@xt@9bp{\hss{\small\textbullet\textbullet}\hss}}}
\def\output@orcid{\lower2\p@\hbox{\href{\orcid@base \@orcid}{\orcid@pic}}}

% Some macros
\newtoks\@temptokenb
\long\def\append@item#1\to#2{%
    \@temptokena={\@k@p{#1}}%
    \@temptokenb=\expandafter{#2}%
    \xdef#2{\the\@temptokenb\the\@temptokena}}
\def\get@left#1\to#2{\expandafter\g@l#1\g@l#1#2}
\long\def\g@l\@k@p#1#2\g@l#3#4{\def#4{#1}\def#3{#2}}

\newcount\cnt@authors
\newcount\cnt@institutes

\def\@authors{}%
\def\@allauthors{}%
\def\@institutes{}%
\def\@instituteauthors{}%
\def\@curauths{}
\def\@curinst{}

\def\authorindent{0pt}
\def\afterallauthorsskip{1em\relax}
\def\afterauthorskip{0pt}

\def\author@#1{\append@item#1\to\@authors\append@item#1\to\@allauthors }

\let\saved@author@\author@
\def\author@{\@ifnextchar[{\@author@}{\@author@[]}}
\def\@author@[#1]#2{\saved@author@{[#1]{#2}}}

\RequirePackage{keyval}
\let\CheckedBox\relax
\RequirePackage{marvosym}
\let\@@Letter\Letter
\def\@@@Letter{\lower.32\p@\hbox{\fontsize{13}{13}\selectfont\@@Letter}\hskip-1.3\p@}

\def\authorfont{\sffamily\normalsize\raggedrightmargin{10mm}\let\sep=\authorsep}
\protected\def\authorsep{~{\mathversion{bold}\ensuremath{\cdot}}\ }

\newcounter{affref}
\def\theaffref{\arabic{affref}}
%% format footnote mark in author block
\def\addressref@fmt#1{\textsuperscript{\addressref@sep #1}}
%% format footnote label in addresses list
\def\addresslbl@fmt#1{\textsuperscript{#1}}

\newif\if@authorcorref
\define@key{author}{corref}[true]{\@authorcorreftrue}%
\define@key{author}{email}{\def\@author@email{#1}}%
\define@key{author}{addressref}{\def\@author@addressref{#1}}

\def\email@prefix{e-mail:~}
% structured names
\def\surname#1{#1}%
\def\inits#1{}%
\def\fnm#1{#1}%
\def\lnm#1{#1}%
\let\sep\relax

\def\@formatauthors{\begingroup
    \authorfont
    \leavevmode
    \gdef\surname##1{##1}%
    \gdef\email##1{\hbox{(\email@prefix{\lowercase{##1}})}}%
    \cnt@authors=0
    \def\@k@p##1{\advance\cnt@authors by 1}\@curauths
    \def\@k@p##1{\advance\cnt@authors by -1
        \ifnum\cnt@authors>1
        \@fmt@author ##1\sep
        \fi
        \ifnum\cnt@authors=1
        \@fmt@author ##1\sep
        \fi
        \ifnum\cnt@authors<1
        \@fmt@author ##1\par
        \fi
    }\@curauths
    \vskip \afterauthorskip
    \printaddresses
    \endgroup
}

\newtoks\corref@authors@list
\newtoks\email@authors@list
\def\fmt@author@and@email#1#2{#1\\{\href{mailto:#2}{\lowercase{#2}}}}
\def\add@xtok#1#2{\begingroup\xdef\@act{\global\noexpand#1{\the#1#2}}\@act\endgroup}
\def\fmt@addressref@list{%
    \ifx\addrespresented\relax
    \else
    \@for\@id:=\@author@addressref\do{%
        \@ifundefined{affref@\@id}%
        {%
            \stepcounter{affref}%
            \expandafter\xdef\csname affref@\@id\endcsname{\theaffref}%
        }%
        {}%
        \addressref@fmt{\csname affref@\@id\endcsname}%
        \def\addressref@sep{,}%
    }%
    \fi
}

\newcommand\@fmt@author[2][]{%
    \bgroup
    \let\inits\relax
    \let\fnm\relax
    \let\@orcid\relax
    \let\@author@email\@empty
    \let\@author@addressref\@empty
    \let\addressref@sep\@empty
    \setkeys{author}{#1}%
    \ifx\@author@email\@empty
    \else
    \if@authorcorref
%    \add@xtok\corref@authors@list{\noexpand\item[\noexpand\@@@Letter]\noexpand\fmt@author@and@email{Corresponding author}{\@author@email}}%
    \add@xtok\corref@authors@list{\noexpand\item[\noexpand\@@@Letter]\noexpand\fmt@author@and@email{#2}{\@author@email}}%
    \else
    \add@xtok\email@authors@list{\noexpand\item[] \noexpand\fmt@author@and@email{#2}{\@author@email}}%
    \fi
    \fi
    \def\inits##1{\ignorespaces}%
    \def\fnm##1{##1\ignorespaces}%
    \noindent #2\fmt@addressref@list
    \ifx\@orcid\relax
    \else
    \output@orcid
    \fi
    \egroup
}

\def\institutesize{\footnotesize\it}
\def\institutecase#1{#1}
\def\instituteindent{0pt}
\def\institutesep{.4\baselineskip}
\def\institutesize{\footnotesize\normalfont}
\def\@formatinstitute{\insert\footins{\institutesize\institutecase{\@curinst}\par}}
\def\institute@#1{\append@item#1\to\@institutes
    \expandafter\append@item\expandafter{\@authors}\to\@instituteauthors
    \gdef\@authors{}}
\newif\ifthanks
\def\orf@#1{#1}
\def\@formatname#1#2{\begingroup
    \def\thanks##1{\global\thankstrue}\setbox0\vbox{#1}%
    \endgroup
    \begingroup
    \ifthanks
    \edef\fn##1\thanks##2{\authorcase{##1}\/%
        \noexpand\thanks{##2}#2}\expandafter\fn#1
    \else
    \edef\fn##1{{\authorcase{##1}}#2}%
    \expandafter\fn\expandafter{#1}%
    \fi
    \endgroup\global\thanksfalse }

\def\fmt@address#1{#1}
\def\make@address@mark#1{%
    \@ifundefined{#1}{??}{\csname #1\endcsname}%
}
\define@key{address}{id}{\def\@address@id{#1}}%
\let\addrespresented\relax
\newtoks\adresses@list
\newcommand\address[2][]{%
    \bgroup
    \global\let\addrespresented\@empty
    \let\@address@id\@empty
    \setkeys{address}{#1}%
    \ifx\@address@id\@empty
    \def\item@mark{}%
    \else
    \def\item@mark{\noexpand\make@address@mark{affref@\@address@id}}%
    \fi
    \add@xtok\adresses@list{\noexpand\item[\item@mark]\noexpand\fmt@address{#2}}%
    \egroup
    \ignorespaces
}

% castead
\def\casteadlistsetup#1{#1}
\def\castaddresseslistsetup#1{#1}
\def\@casteadlistsetup{%
    \labelwidth15pt
    \itemindent\labelwidth
    \parindent\itemindent
    \itemsep 1pt
}

\def\@casteadlisthook{%
    \def\makelabel##1{\hfil ##1}%
}

\def\castead{%
    \bgroup
    \let\orcid\@gobble
    \let\make@trivlist\@firstofone
    %% do nothing if lists are empty
    \expandafter\def\expandafter\@temp\expandafter{\the\corref@authors@list}%
    \ifx\@temp\@empty
    \expandafter\def\expandafter\@temp\expandafter{\the\email@authors@list}%
    \ifx\@temp\@empty
    \let\make@trivlist\@gobble
    \fi
    \fi
    \def\fnm##1{\ignorespaces}%
    \def\inits##1{##1\ignorespaces}%
    \make@trivlist{%
        \csname castead@skip\endcsname
        \trivlist
        \@casteadlisthook
        \@casteadlistsetup
        \the\corref@authors@list
        \the\email@authors@list
        \endtrivlist
        \global\email@authors@list={}%
        \global\corref@authors@list={}%
    }%
    \egroup
}

% castaddresses
\def\castaddresses@skip{\addvspace{0\p@}}
\def\@castaddresseslistsetup{%
    \topsep0pt
    \parskip0pt
    \partopsep0pt
    \listparindent0pt
    \itemindent0pt
    \labelsep0pt
    \leftmargin15pt
    \labelwidth\leftmargin
    \itemsep 1pt
}

\def\@castaddresseslisthook{%
    \def\makelabel##1{\rlap{\addresslbl@fmt{##1}}}%
}

\def\castaddresses{%
    \bgroup
    \let\make@list\@firstofone
    %% do nothing if lists are empty
    \expandafter\def\expandafter\@temp\expandafter{\the\adresses@list}%
    \ifx\@temp\@empty
    \let\make@list\@gobble
    \fi
    \make@list{%
        \castaddresses@skip
        \list{}{\@castaddresseslistsetup}%
        \@castaddresseslisthook
        \the\adresses@list
        \endlist
    }
    \egroup
}


% -- cmd::printaddresses
\global\let\orig@footnoterule\footnoterule
\def\fminsert@size{\footnotesize\normalfont\raggedrightmargin{5mm}}
\def\footnoterule@width{108\p@}
\long\def\printaddresses{%
    \gdef\footnoterule{\global\let\footnoterule\orig@footnoterule}%
    \insert\footins{%
        \fminsert@size
        \vskip2.7\p@
        \hrule width\footnoterule@width
        \vspace*{-1.5ex}
        \castead
        \vspace*{-1.5ex}
        \castaddresses
        \par
    }%
}

% -- cmd::authorsandinstitutes
\def\@authorsandinstitutes{\begingroup
    \authorfont
    \cnt@authors=0
    \def\@k@p##1{\advance\cnt@authors by 1}\@allauthors
    \cnt@institutes=0
    \def\@k@p##1{\advance\cnt@institutes by 1}\@institutes\relax
    \ifnum\cnt@institutes=0
    \let\@curauths\@allauthors
    \parindent=\authorindent
    \hangindent=\authorindent
    \@formatauthors
    \fi
    \loop\ifnum\cnt@institutes>0
    \get@left\@instituteauthors\to\@curauths
    \parindent=\authorindent
    \hangindent=\authorindent
    \@formatauthors
    \get@left\@institutes\to\@curinst
    \parindent=\instituteindent
    \hangindent=\instituteindent
    \@formatinstitute
    \ifnum\cnt@institutes>1 \vskip \institutesep\relax \fi
    \advance\cnt@institutes by -1
    \repeat
    \vskip \afterallauthorsskip
    \gdef\@authors{}%
    \gdef\@allauthors{}%
    \gdef\@institutes{}%
    \gdef\@instituteauthors{}%
    \gdef\@curauths{}%
    \gdef\@curinst{}%
    \endgroup}

% -- cmd::date
\def\afterdateskip{.0\baselineskip}
\def\datesize{\footnotesize\sffamily}
\def\@date{%
    {\datesize\@received~/ \@revised~/ \@accepted~/ \@published}\vskip \afterdateskip
}
\def\date@#1{%
    \gdef\@date{{\datesize #1\par}\vskip \afterdateskip}
}
\def\@received{}
\def\@revised{}
\def\@accepted{}
\def\@published{}
\def\received@#1{\gdef\@received{Received:~#1}}
\def\revised@#1{\gdef\@revised{Revised:~#1}}
\def\accepted@#1{\gdef\@accepted{Accepted:~#1}}
\def\published@#1{\gdef\@published{\\[-2\p@] Published Online:~#1}}


% -- cmd::copyright
\def\copyright@font{\footnotesize\sffamily}
\def\copyright@text{\textcopyright\ Operations Research Society of China, Periodicals Agency of Shanghai University, Science Press, and Springer-Verlag GmbH Germany, part of Springer Nature \the\year}
\def\sprcopyright{\gdef\@copyright{\copyright@font\copyright@text\vskip2\baselineskip}}
% execute
\sprcopyright


% -- env::abstract
\def\abstractnamefont{\sffamily\bfseries\normalsize}
\def\abstractdot{\par}
\def\abstractname{Abstract}
\def\afterabstractskip{.7\baselineskip}
\def\preabstractskip{\z@}
\def\abstract#1{%
    \linenumbershook
    \vskip\preabstractskip
    {\abstractnamefont\noindent\abstractname}\abstractdot
    {\rmfamily\noindent\ignorespaces #1}\par
    \vskip\afterabstractskip
}{}

% -- cmd::keywords
\def\keynamefont{\sffamily\bfseries}
\def\keywordsname{Keywords}
% \def\@keywords{}
\long\def\keywords#1{%
    % \gdef\@keywords{%
    \noindent{\keynamefont\keywordsname~}{\rmfamily #1}\par \vskip.7\baselineskip
}

% -- cmd::classification
\def\classnamefont{\sffamily\bfseries}
\def\classificationname{Mathematics Subject Classification}
\long\def\classification#1{\noindent{\classnamefont\classificationname~}{\rmfamily #1}\par \vskip.7\baselineskip}

% -- cmd::maketitle
\def\@maketitle{%
    \@updateicon
    \@title
    \@authorsandinstitutes
    %\@date
    %\@copyright
    % \@abstract
    % \@keywords
    % \@classification
}

\edef\@checkurl{http://crossmark.crossref.org/dialog/?doi=\@thedoi\&domain=pdf}
\def\@checkicon{}%\includegraphics[width=10mm,height=10mm]{crossmark}}
\def\@updateicon{\hfill \href{\@checkurl}{\@checkicon}\par\nointerlineskip}

\def\openingflushstyle{}
\def\maketitle@@{%
    \begingroup
    \setcounter{footnote}{0}%
    \def\thefootnote{\fnsymbol{footnote}}%
    \if@twocolumn
    \twocolumn[{\openingflushstyle \parindent 0pt \@maketitle}]
    \thispagestyle{opening}%
    \@extramaketitle
    \else
    \newpage\global\@topnum\z@
    \thispagestyle{opening}%
    {\openingflushstyle \parindent 0pt \@maketitle}%
    \fi
    \markboth{\@runningauthor}{\@runningtitle}%
    \@thanks
    \endgroup
    \setcounter{footnote}{0}%
    \let\@maketitle\relax
    \gdef\@thanks{}%
    \gdef\@title{}%
    \let\thanks\relax
}

% -- cmd::runningtitle/runningauthor
\def\runningtitle#1{\gdef\@runningtitle{#1}}
\gdef\@runningtitle{}
\def\runningauthor#1{\gdef\@runningauthor{#1}}
\gdef\@runningauthor{}
%

% -- cmd::insertfoot
\def\insertfoot#1{
\insert\footins{%
\vskip3.7\p@
\hrule width\footnoterule@width
\vskip4.2\p@
\footnotesize\noindent #1}\par
}


% ==== Geometry settings
\RequirePackage[%
paperwidth=155mm,
paperheight=235mm,
text={119mm,190mm},
%marginparwidth=4cm,%
%marginparsep=0.5cm,%
right=18mm,%
top=20mm,%
%bottom=2.5cm,%
%headheight=21pt,%
headsep=\baselineskip,%
%footskip=2\baselineskip,%
%columnsep=2em,%
%showframe
]{geometry}%


% ==== sectioning
\def\@seccntformat#1{\csname the#1\endcsname\hskip.5em}
\def\nohyphen{\pretolerance=\@M \tolerance=\@M \hyphenpenalty=\@M \exhyphenpenalty=\@M}
\renewcommand\section{\@startsection{section}{1}{\z@}%
    {-\bigskipamount}%
    {\medskipamount}%
    {\fontsize{11}{12.5}\bfseries\sffamily\mathversion{bold}\raggedright\nohyphen}}
\renewcommand\subsection{\@startsection{subsection}{2}{\z@}%
    {-\medskipamount}%
    {\medskipamount}%
    {\fontsize{10}{12.5}\bfseries\sffamily\mathversion{bold}\raggedright\nohyphen}}
\renewcommand\subsubsection{\@startsection{subsubsection}{3}{\z@}%
    {-\medskipamount}%
    {\medskipamount}%
    {\fontsize{10}{12.5}\bfseries\sffamily\raggedright}}
\renewcommand\paragraph{\@startsection{paragraph}{4}{\z@}%
    {\medskipamount}%
    {-10pt}%
    {\bfseries\mathversion{bold}}}
\renewcommand\subparagraph{\@startsection{subparagraph}{5}{\parindent}%
    {0.1pt}%
    {-1em}%
    {\itshape}}

% ==== appendix
\RequirePackage[title]{appendix}%
\renewcommand\appendixname{Appendix}%
\@ifpackageloaded{appendix}{%
%
\renewenvironment{appendices}{%
  \@resets@pp
  \if@dotoc@pp
    \if@dopage@pp              % both page and toc
      \if@chapter@pp           % chapters
        \clear@ppage
      \fi
      \appendixpage
    \else                      % toc only
       \if@chapter@pp          % chapters
         \clear@ppage
       \fi
      \addappheadtotoc
    \fi
  \else
    \if@dopage@pp              % page only
      \appendixpage
    \fi
  \fi
  \if@chapter@pp
    \if@dotitletoc@pp \@redotocentry@pp{chapter} \fi
  \else
    \if@dotitletoc@pp \@redotocentry@pp{section} \fi
    \if@dohead@pp
      \def\sectionmark##1{%
        \if@twoside
          \markboth{\@formatsecmark@pp{##1}}{}
        \else
          \markright{\@formatsecmark@pp{##1}}{}
        \fi}
    \fi
    \if@dotitle@pp
      \def\sectionname{\appendixname}
      \def\@seccntformat##1{\@ifundefined{##1name}{}{\csname ##1name\endcsname\ }%
        \csname the##1\endcsname\quad}
    \fi
  \fi
}{%
  \@ppsaveapp\@pprestoresec}
%%
\AtBeginDocument{%
%
\let\oldappendices\appendices%
\let\oldendappendices\endappendices%
%%
\renewenvironment{appendices}{%
\setcounter{figure}{0}%
\setcounter{table}{0}%
\setcounter{equation}{0}%
%%
\begin{oldappendices}%
  \gdef\thefigure{\@Alph\c@section\arabic{figure}}%
  \gdef\thetable{\@Alph\c@section\arabic{table}}%
  \gdef\theequation{\@Alph\c@section\arabic{equation}}%
}{\end{oldappendices}}
}
%%
}{}


% ==== Math
\RequirePackage{amsmath,amssymb,amsthm}\let\Bbbk\relax
%\let\le\leqslant
%\let\leq\leqslant
%\let\ge\geqslant
%\let\geq\geqslant
% -- env::theorem
\def\@begintheorem#1#2[#3]{%
    \deferred@thm@head{\the\thm@headfont \thm@indent
        \@ifempty{#1}{\let\thmname\@gobble}{\let\thmname\@iden}%
        \@ifempty{#2}{\let\thmnumber\@gobble}{\let\thmnumber\@iden}%
        \@ifempty{#3}{\let\thmnote\@gobble}{\let\thmnote\@iden}%
        \thm@swap\swappedhead\thmhead{#1}{#2}{#3}%
        \the\thm@headpunct
        \thmheadnl % possibly a newline.
        \hskip\thm@headsep
    }%
    \ignorespaces
}
\def\@endtheorem{\endtrivlist\@endpefalse}
\def\thm@space@setup{%
    \thm@preskip=12pt%
    \thm@postskip=12pt}
%
% -- env::proof
% with Qed symbol automatically added
\AtBeginDocument{%
%
\DeclareSymbolFont{AMSa}{U}{msa}{m}{n}%
\DeclareMathSymbol{\opensquare}{\mathord}{AMSa}{"03}%
\def\qedsymbol{}%{\opensquare}%
%
\newenvironment{spiproof}[1][\proofname]{\par%\removelastskip%\vspace*{2pt}%
  \pushQED{\qed}%
  \small\normalfont \topsep7.5\p@\@plus7.5\p@\relax%
  \trivlist%
  \item[\hskip\labelsep%
        \normalsize\bfseries\sffamily%\itshape%
    #1\@addpunct{}]\ignorespaces%
}{%
  \popQED\endtrivlist\@endpefalse%
}%
%
\let\proof\spiproof\let\endproof\endspiproof%
%
}%
%
% -- theorem-like environments
\newtheoremstyle{thmstyleone}% Numbered
{18pt plus2pt minus1pt}% Space above
{18pt plus2pt minus1pt}% Space below
{\normalsize\itshape}% Body font
{0pt}% Indent amount
{\normalsize\bfseries\sffamily}% Theorem head font
{}% Punctuation after theorem head
{.5em}% Space after theorem headi
{\thmname{#1}\thmnumber{\@ifnotempty{#1}{ }\@upn{#2}}%
    \thmnote{ {\the\thm@notefont(#3)}}}% Theorem head spec (can be left empty, meaning `normal')
%
\newtheoremstyle{thmstyletwo}% Numbered
{18pt plus2pt minus1pt}% Space above
{18pt plus2pt minus1pt}% Space below
{\normalsize\normalfont}% Body font
{0pt}% Indent amount
{\normalsize}% Theorem head font
{}% Punctuation after theorem head
{.5em}% Space after theorem headi
{\thmname{\textit{#1}}\thmnumber{\@ifnotempty{#1}{ }{#2}}%
    \thmnote{{\the\thm@notefont(#3)}}}% Theorem head spec (can be left empty, meaning `normal')
%
\newtheoremstyle{thmstylethree}% Definition
{18pt plus2pt minus1pt}% Space above
{18pt plus2pt minus1pt}% Space below
{\normalsize\normalfont}% Body font
{0pt}% Indent amount
{\normalsize\bfseries\sffamily}% Theorem head font
{}% Punctuation after theorem head
{.5em}% Space after theorem headi
{\thmname{#1}\thmnumber{\@ifnotempty{#1}{ }\@upn{#2}}%
    \thmnote{ {\the\thm@notefont(#3)}}}% Theorem head spec (can be left empty, meaning `normal')
%
\newtheoremstyle{thmstylefour}% Proof
{18pt plus2pt minus1pt}% Space above
{18pt plus2pt minus1pt}% Space below
{\normalsize\normalfont}% Body font
{0pt}% Indent amount
{\normalsize\itshape}% Theorem head font
{}% Punctuation after theorem head
{.5em}% Space after theorem headi
{\global\proofthmtrue\thmname{#1} \thmnote{#3}}% Theorem head spec (can be left empty, meaning `normal')

\theoremstyle{thmstyleone}%
\newtheorem{theorem}{Theorem}% meant for continuous numbers
\newtheorem{lemma}{Lemma}%
\newtheorem{proposition}{Proposition}

\theoremstyle{thmstyletwo}%
%\newtheorem{example}{Example}%

\theoremstyle{thmstylethree}%
\newtheorem{remark}{Remark}%
\newtheorem{example}{Example}%
\newtheorem{definition}{Definition}%

% ==== Bibliography
\def\bibcommenthead{}%
\RequirePackage[numbers,sort&compress]{natbib}%
%\RequirePackage[authoryear,sort&compress,round,comma]{natbib}%

%\bibliographystyle{camc}%
\setlength{\bibsep}{.5ex}%
\def\bibfont{\reset@font\fontfamily{\rmdefault}\footnotesize\selectfont}%
\renewcommand\refname{References}%

% ==== Float
\RequirePackage{graphicx}
\RequirePackage{xcolor}
\ifxetexorluatex\else%
\RequirePackage{epstopdf}\fi
\RequirePackage{subfig}
\RequirePackage[figuresright]{rotating}
\RequirePackage{booktabs}
% caption style
\captionsetup[figure]{%
name=Fig.,
format=plain,
singlelinecheck=false,
justification=raggedright,
labelfont={small,sf},
textfont=small,
labelsep=quad}
\captionsetup[subfigure]{%
format=plain,
singlelinecheck=false,
justification=centering,
labelfont={small},
textfont=small,
labelsep=space}
\captionsetup[table]{%
name=Table,
singlelinecheck=false,
justification=raggedright,
labelfont={small,sf},
textfont=small,
labelsep=quad,
skip=.5ex}
% three-line table rule width
\@ifpackageloaded{booktabs}{\cmidrulewidth=.2pt}{}%
% customize table style
\def\tablecaptionfont{\reset@font\fontsize{8bp}{9.5bp}\selectfont}%
\def\tablebodyfont{\reset@font\fontsize{8bp}{9.5bp}\selectfont}%
\def\tablecolheadfont{\reset@font\fontsize{8bp}{9.5bp}\selectfont\bfseries\boldmath}%
\def\tablefootnotefont{\reset@font\fontsize{8bp}{9.5bp}\selectfont}%
\let\tableorg\table%
\let\endtableorg\endtable%
\renewenvironment{table}[1][]%
{\begin{tableorg}[#1]%
        \tablebodyfont%
        \renewcommand\footnotetext[2][]{{\removelastskip\vskip3pt%
                \let\tablebodyfont\tablefootnotefont%
                \hskip0pt\if!##1!\else{\smash{$^{##1}$}}\fi##2\par}}%
    }{\end{tableorg}}
% customize sidewaystable style
\let\sidewaystableorg\sidewaystable%
\let\endsidewaystableorg\endsidewaystable%
\renewenvironment{sidewaystable}%
{\begin{sidewaystableorg}%
     \tablebodyfont%
     \renewcommand\@makefntext[1]{{\vskip3pt%
             \let\tablebodyfont\tablefootnotefont%
             \hskip0pt{\smash{\@makefnmark}}##1\par}}%
     }{\end{sidewaystableorg}}
\def\sidewaystablefn{\renewcommand\footnotetext[2][]{{\removelastskip\vskip3pt%
            \let\tablebodyfont\tablefootnotefont%
            \hskip0pt\if!##1!\else{\smash{$^{##1}$}}\fi##2\par}}%
}%


% ==== Algorithm
\RequirePackage{algorithm}%
\RequirePackage{algorithmicx}%
\RequirePackage{algpseudocode}%
\RequirePackage{program}%
% set caption style
\captionsetup[algorithm]{%
name=Algorithm,
format=plain,
singlelinecheck=false,
labelfont=bf}
% set rules off
\def\fs@ruled@notop{%
    \gdef\@fs@cfont{\bfseries\rmfamily}\let\@fs@capt\floatc@ruled
    % \def\@fs@pre{}%{\hrule height.8pt depth0pt \kern2pt}
    \def\@fs@pre{}%
    \def\@fs@post{}%{\kern2pt\hrule\relax}%
    \def\@fs@mid{\null\vskip\z@}%{\kern2pt\hrule\kern2pt}%
    \let\@fs@iftopcapt\iftrue}
\renewcommand\fst@algorithm{\fs@ruled@notop}
\def\ALG@step%
{%
    \addtocounter{ALG@line}{1}%
    \addtocounter{ALG@rem}{1}%
    \ifthenelse{\equal{\arabic{ALG@rem}}{\ALG@numberfreq}}%
    {\setcounter{ALG@rem}{0}\alglinenumber{\roman{ALG@line}}}%
    {}%
}%
%\RequirePackage{xpatch}
%\xpatchcmd{\algorithmic}% <cmd>
%{\arabic{ALC@line}}% <search>
%{\theALC@line}% <replace>
%{}{}% <success><failure>
%%\xpatchcmd{\algorithmic}{1.2em}{1.5em}{}{}% Increase label width
\renewcommand\theALG@line{\roman{ALG@line}}


%% ==== line numbers
\if@linenumberon%
\RequirePackage[mathlines,running]{lineno}
\let\linenumbershook\linenumbers
%\AtBeginEnvironment{abstract}{\begingroup\linenumbers}
%\AtEndDocument{\endlinenumbers\endgroup}
\else
\let\linenumbershook\empty
\fi%


%% ==== Watermark
% !!! may cause clash with \pkg{hyperref}, need fix
\if@watermarkon
\AtEndPreamble{%
\RequirePackage[fontsize=0.22\paperwidth,angle=60,text=Revised~Proof]{draftwatermark}
}
\fi

\AtEndPreamble{%
    \RequirePackage{indentfirst}
    \RequirePackage{hyperref}
    \hypersetup{%
    bookmarks=true,         % show bookmarks bar?
    unicode=false,          % non-Latin characters in Acrobat’s bookmarks
    pdftoolbar=true,        % show Acrobat’s toolbar?
    pdfmenubar=true,        % show Acrobat’s menu?
    pdffitwindow=false,     % window fit to page when opened
    pdfstartview={FitH},    % fits the width of the page to the window
    pdftitle={},            % title
    pdfauthor={},           % author
    pdfsubject={\@jname},   % subject of the document
    pdfcreator={\@springer},     % creator of the document
    pdfproducer={},              % producer of the document
    pdfkeywords={}, % list of keywords
    pdfnewwindow=true,        % links in new window
    pdfborder={0 0 0},
    colorlinks=false,
    linkcolor=black,
    citecolor=black,
    urlcolor=blue,
    breaklinks}
    % line breakable within url
    \def\UrlBreaks{%
        \do\/%
        \do\a\do\b\do\c\do\d\do\e\do\f\do\g\do\h\do\i\do\j\do\k\do\l%
        \do\m\do\n\do\o\do\p\do\q\do\r\do\s\do\t\do\u\do\v\do\w\do\x\do\y\do\z%
        \do\A\do\B\do\C\do\D\do\E\do\F\do\G\do\H\do\I\do\J\do\K\do\L%
        \do\M\do\N\do\O\do\P\do\Q\do\R\do\S\do\T\do\U\do\V\do\W\do\X\do\Y\do\Z%
        \do0\do1\do2\do3\do4\do5\do6\do7\do8\do9\do=\do/\do.\do:%
        \do\*\do\-\do\~\do\'\do\"\do\-}
    \Urlmuskip=0mu plus 0.1mu
    \urlstyle{sf}
    \AtBeginEnvironment{thebibliography}{\urlstyle{rm}}
}
